' musicxml2tex --- Un convertisseur MusicXML vers MusixTeX
' Copyright 2014 Vincent Belaïche
'
' Author: Vincent Belaïche
'         e-mail : vincent.b.1@hotmail.fr
'
' Version: $Id: musicxml2tex.vbs 13 2014-12-16 22:12:28Z vincentb1 $
' Keywords: main
' X-URL: https://gna.org/projects/musicxml2tex/
'
' Ce logiciel est régi par la licence CeCILL soumise au droit français et respectant les principes de
' diffusion des logiciels libres. Vous pouvez utiliser, modifier et/ou redistribuer ce programme sous les
' conditions de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA sur le site
' "http://www.cecill.info".
'
' En contrepartie de l'accessibilité au code source et des droits de copie, de modification et de
' redistribution accordés par cette licence, il n'est offert aux utilisateurs qu'une garantie limitée.  Pour
' les mêmes raisons, seule une responsabilité restreinte pèse sur l'auteur du programme, le titulaire des
' droits patrimoniaux et les concédants successifs.
'
' À cet égard l'attention de l'utilisateur est attirée sur les risques associés au chargement, à
' l'utilisation, à la modification et/ou au développement et à la reproduction du logiciel par l'utilisateur
' étant donné sa spécificité de logiciel libre, qui peut le rendre complexe à manipuler et qui le réserve
' donc à des développeurs et des professionnels avertis possédant des connaissances informatiques
' approfondies.  Les utilisateurs sont donc invités à charger et tester l'adéquation du logiciel à leurs
' besoins dans des conditions permettant d'assurer la sécurité de leurs systèmes et ou de leurs données et,
' plus généralement, à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
'
' Le fait que vous puissiez accéder à cet en-tête signifie que vous avez pris connaissance de la licence
' CeCILL, et que vous en avez accepté les termes.
'
'/ Commentary:

' To get usage type the following command in a cmd.exe console:
'
'  cscript musicxml2tex.vbs --help



'/ Code:
Option Explicit
Call EnsureCScript

Const sVersion = "0.0"

Class CMusicDocument
	Public sMovementTitle
	Public iErrorFlag
	Public iWarningCount
	
	Private Sub LoadMusicXML(sInputFileName_IN)
     Dim oXMlDoc
     Set oXMlDoc = WScript.CreateObject("Microsoft.XMLDOM")
	  oXMlDoc.async = False
	  oXMlDoc.validateOnParse = False
     oXMlDoc.Load sInputFileName_IN

	  Dim oNodeCollection, oNode
	  Set oNodeCollection = oXMlDoc.getElementsByTagName("score-partwise")
	  If oNodeCollection.length = 0 Then
		  WScript.Echo "Erreur : ""score-partwise"" absent"
		  iErrorFlag = -1
		  Exit Sub
	  ElseIf oNodeCollection.length > 1 Then
		  WScript.Echo "Avertissement : ""score-partwise"" multiple"
		  iWarningCount = iWarningCount + 1
	  End If
		  
	  Dim oScorePartwiseNode
	  Set oScorePartwiseNode = oNodeCollection.Item(0)

	  Set oNodeCollection = oScorePartwiseNode.getElementsByTagName("movement-title")
	  If oNodeCollection.length = 0 Then
		  WScript.Echo "Avertissement : ""movement-title"" absent"
		  iWarningCount = iWarningCount + 1
	  Else
		  If oNodeCollection.length > 1 Then
			  WScript.Echo "Avertissement : ""movement-title"" multiple"
			  iWarningCount = iWarningCount + 1
		  End If
		  Set oNode = oNodeCollection.Item(0)
		  sMovementTitle = oNode.text
	  End If
		  

	  Set oXMlDoc = Nothing
	  Set oNode = Nothing
	  Set oNodeCollection = Nothing
	  iErrorFlag = 0
	End Sub
	
	Public Sub LoadInput(sInputFileName_IN, sInputFormat_IN)
		Select Case sInputFormat_IN
			Case "musicxml" : LoadMusicXML(sInputFileName_IN)
			Case Else
				WScript.Echo "Format d'entrée invalide: " & sInputFormat_IN
				WScript.Quit(-1)
		End Select
	
	End Sub

	Sub Class_Initialize
		iWarningCount = 0
	End Sub
End Class

Const MX2T_iBP_CLOSING_CURLY = 0
Const MX2T_iBP_CS = 1
Const MX2T_iBP_NUMBER = 2
Const MX2T_iBP_OTHER = 3

Class CMusicTexFormatter

	Private oOut
	Private iBeforePoint
	
	Private Sub FormatCS(sCsName_IN,oArgArray_IN)
		oOut.Write "\" & sCsName_IN
		iBeforePoint = MX2T_iBP_CS
		If IsNull(oArgArray_IN) Then
			Exit Sub
		Else
			Dim oArg
			For Each oArg In oArgArray_IN
				Dim sArg
				sArg = CStr(oArg)
				If Len(sArg) = 0 Then
					oOut.Write "{}"
					iBeforePoint = MX2T_iBP_CLOSING_CURLY
				ElseIf Len(sArg) = 1 Then
					iOut.Write sArg
					If IsInteger(oArg) Then
						iBeforePoint = MX2T_iBP_NUMBER
					Else
						iBeforePoint = MX2T_iBP_OTHER
					End If
				Else
					oOut.Write "{" & sArg & "}"
					iBeforePoint = MX2T_iBP_CLOSING_CURLY
				End If
			Next
		End If
	End Sub
	
	Public Property Let OuputStream(sOutputFileName_IN)
		If Not IsEmpty(oOut) Then
			oOut.Close
		End If 
		Dim oFSO
		Set oFSO = WScript.CreateObject("Scripting.FileSystemObject")
		Set oOut = oFSO.CreateTextFile(sOutputFileName_IN, True)
		Set oFSO = Nothing
		oOut.WriteLine "% This file was generated automatically"
		oOut.WriteLine "% Creator: musicxml2tex.vbs"
	End Property

	Private Sub FormatPreamble(oMusicDocument_IN)
		oOut.WriteLine "\input musixtex.tex"
		oOut.WriteLine "\input musixsty.tex"
		oOut.WriteLine "\input musixps"
		oOut.WriteLine "\nohalfties"
		oOut.WriteLine "\def\pstiehgt{0.5}"
		oOut.WriteLine "\def\psslurhgt{0.5}"
		oOut.WriteLine ""
		oOut.WriteLine "\vsize9.75in"
		oOut.WriteLine "\voffset-0.5in"
		oOut.WriteLine "\hsize7.0in"
		oOut.WriteLine "\hoffset-0.25in"
		oOut.WriteLine "\smallmusicsize"
		oOut.WriteLine "\bigaccid"
		oOut.WriteLine "\advance\parindent by 10pt"
		oOut.WriteLine "\def\raisebarno{7\internote}%"
		oOut.WriteLine "\nobarnumbers"
		oOut.WriteLine "\geometricskipscale"
		oOut.WriteLine ""
		oOut.WriteLine "\def\tql#1#2{\tbl{#1}\qb{#1}{#2}}"
		oOut.WriteLine "\def\tqu#1#2{\tbu{#1}\qb{#1}{#2}}"
		oOut.WriteLine "\def\tqql#1#2{\roffset{0.25}{\tbbl{#1}\tbl{#1}\qb{#1}{#2}}}"
		With oMusicDocument_IN
   		Call FormatCS("fulltitle",Array(.sMovementTitle))
   		Call FormatCS("title",Array(.sMovementTitle))
		End With
		Call FormatCS("maketitle",Array())
	End Sub

	Private Sub FormatPostamble(oMusicDocument_IN)
		oOut.WriteLine "\setrightrepeat%"
		oOut.WriteLine "\mulooseness=0%"
		oOut.WriteLine "\endpiece%"
		oOut.WriteLine "\eject"
		oOut.WriteLine "\end"
	End Sub
	
	Public Sub Format(oMusicDocument_IN)
		Call FormatPreamble(oMusicDocument_IN)
		Call FormatPostamble(oMusicDocument_IN)
	End Sub

	Private Sub Class_Terminate()
		If Not IsEmpty(oOut) Then
			oOut.Close
		End If
	End Sub

	Private Sub Class_Initialize()
		iBeforePoint = MX2T_iBP_CLOSING_CURLY
	End Sub
	
End Class

Dim iArgNb
iArgNb = 0

Dim oArgs
Set oArgs = WScript.Arguments

Dim sOutputName, sInputFormat
' Valeurs par défaut
sOutputName = Null
sInputFormat = "musicxml"
Dim bNextArgs
bNextArgs = True
Do
	If iArgNb = oArgs.Count Then
		WScript.Echo "Nom du fichier d'entrée non spécifié"
		SyntaxError()
	End If
	Select Case oArgs.Item(iArgNb)
		Case "-h", "--help" : Call Help
		Case "-o", "--output"
			iArgNb = iArgNb + 1
			If iArgNb = oArgs.Count Then
				WScript.Echo "Nom du fichier de sortie non spécifié"
				SyntaxError()
			End If
			sOutputName = oArgs.Item(iArgNb)
			iArgNb = iArgNb + 1
		Case Else : bNextArgs = False
	End Select
Loop	While bNextArgs


Dim sInputName
sInputName = oArgs.Item(iArgNb)
If IsNull(sOutputName) Then
	If Right(sInputName,4) = ".xml" Then
		sOutputName = Left(sInputName,Len(sInputName)-4) & ".tex"
	Else
		sOutputName = sInputName & ".tex"
	End If
End If

Dim oMusicDocument
Set oMusicDocument = New CMusicDocument

oMusicDocument.LoadInput sInputName, sInputFormat

Dim oFormatter
Set oFormatter = New CMusicTexFormatter
oFormatter.OuputStream = sOutputName
Call oFormatter.Format(oMusicDocument)


Set oMusicDocument = Nothing
Set oFormatter = Nothing




Sub Help

   WScript.Echo "musicxml2tex Help" & vbCrLF & _ 
			 "Current Version : $version" & vbCrLF & _
			 "usage : musicxml2tex [options] <musicxml filename>" & vbCrLF & _
			 "musicxml2tex can be used to construct a LaTeX2e musictex or " & _
			 "musixtex file file from the specified MusicXML file <musicxml filename>" & vbCrLF & _
			 "Optional Arguments:" & vbCrLF & _
			 "-o <output filename>" & vbCrLF & _
			 "Set the output file name to <output filename>, otherwise it is set by " & _
			 "default by replacing the extention of <musicxml filename> with " & _
			 """.tex""" & vbCrLF & _
			 "--output <output filename>" & _
			 "same as -o"   & vbCrLF & _
			 "-h                 : help message" & vbCrLF & _
			 "-v                 : verbose" & vbCrLF 


   WScript.Quit
End Sub



Sub SyntaxError
   WScript.Echo "Syntax : musicxml2tex [options] <musicxml-file>" & vbCrLF & "Use -h for help" 
   WScript.Quit(-1)
End Sub

Sub EnsureCScript
End Sub

'' end of file musicxml2tex.vbs

' Local Variables:
' compile-command: "make -C ../test vbscript_output/TrioPourPiano.tex"
' coding: utf-8
' End:
